import java.awt.Color;
import java.awt.event.KeyEvent;
import java.util.Random;

import javax.swing.ImageIcon;

/**
 * Driver for game applications.
 * 
 * @author CS159 Instructors
 * @version s23
 */
public class GameDriver {

    public static final Color SCREEN_COLOR = new Color(0, 0, 0);
    public static final int DRAW_DELAY = 10; // in milliseconds
    public static final int SCREEN_WIDTH = 480;
    public static final int SCREEN_HEIGHT = 480;

    // Game objects should use this random number generator rather than using
    // different generators across multiple classes. This approach facilitates
    // reproducible testing by setting an initial seed for this generator.
    public static final Random GENERATOR = new Random(0);

    /**
     * Create a game object and a display screen, and execute the main update and draw loop.
     * 
     * @param args command-line arguments
     */
    public static void main(String[] args) {

        // initialize the game screen
        StdDraw.setCanvasSize(SCREEN_WIDTH, SCREEN_HEIGHT);
        StdDraw.setXscale(0, SCREEN_WIDTH);
        StdDraw.setYscale(0, SCREEN_HEIGHT);
        StdDraw.enableDoubleBuffering();

        // create a game object
        // Playable game = new BounceGame();
        Playable game = new AsteroidsGame();
        game.startGame();

        // main game loop
        while (true) {
            StdDraw.clear(SCREEN_COLOR);
            game.update();
            game.draw();
            StdDraw.show();
            StdDraw.pause(DRAW_DELAY);
        }

    }

    public static Character charPressed() {
        Character pressed = null;
        boolean hasNext = StdDraw.hasNextKeyTyped();
        if (hasNext) {
            pressed = StdDraw.nextKeyTyped();
        }
        return pressed;
    }

    public static boolean leftPressed() {
        return StdDraw.isKeyPressed(KeyEvent.VK_LEFT);
    }

    public static boolean rightPressed() {
        return StdDraw.isKeyPressed(KeyEvent.VK_RIGHT);
    }

    public static boolean upPressed() {
        return StdDraw.isKeyPressed(KeyEvent.VK_UP);
    }

    /**
     * Compute the width and height of the specified image such that it is scaled proportionately to fit within the
     * specified radius.
     * 
     * @param imgPath path to an image
     * @param radius the maximum value to permit for the width and height of the image
     * @return an array of length 2 where the element at position: 0 is the scaled width and 1 is the scaled height
     */
    public static double[] scaledWidthHeight(String imgPath, double radius) {
        double[] scaledWH = new double[2];
        ImageIcon tmp = new ImageIcon(imgPath);
        double maxImgDimension = Math.max(tmp.getIconWidth(), tmp.getIconHeight());
        double factor = 1.0 / maxImgDimension;
        scaledWH[0] = tmp.getIconWidth() * factor * radius;
        scaledWH[1] = tmp.getIconHeight() * factor * radius;
        return scaledWH;
    }

    /**
     * Compute the heading that aims from a to b.
     * 
     * @param a Point to aim from
     * @param b Point to aim at
     * @return heading that points from a to b
     */
    public static double headingAB(Point a, Point b) {
        double deltaX = b.getX() - a.getX();
        double deltaY = b.getY() - a.getY();
        double towardB = Math.atan(deltaY / deltaX);
        return towardB;
    }
}
