/**
 * Interactive / moving elements of the game.
 * 
 * @author CS159 Instructors
 * @version s23
 */
public abstract class GameElement implements Drawable, Updatable {
    protected Pose pose;
    protected Vector2D velocity;
    protected double radius;
    protected boolean destroyed;

    /**
     * Constructs a game element with the given values.
     * 
     * @param pose initial pose
     * @param velocity initial velocity
     * @param radius collision radius
     */
    public GameElement(Pose pose, Vector2D velocity, double radius) {
        this.pose = pose;
        this.velocity = velocity;
        this.radius = radius;
        this.destroyed = false;
    }

    public Pose getPose() {
        return pose;
    }
    
    public Vector2D getVelocity() {
        return velocity;
    }
    
    public double getRadius() {
        return radius;
    }
    
    /**
     * Determines whether this element collides with the other.
     * 
     * @param other the other element
     * @return true if collision, false otherwise
     */
    public boolean checkCollision(GameElement other) {
        double diffX = this.pose.getX() - other.pose.getX();
        double diffY = this.pose.getY() - other.pose.getY();
        double distance = Math.sqrt(diffX * diffX + diffY * diffY);
        return distance < this.radius + other.radius;
    }

    public boolean isDestroyed() {
        return destroyed;
    }

    public void setDestroyed(boolean destroyed) {
        this.destroyed = destroyed;
    }

    @Override
    public void update() {
        pose = pose.move(velocity);

        // wrap at left/right edge
        if (pose.getX() > GameDriver.SCREEN_WIDTH) {
            pose = pose.newX(0);
        } else if (pose.getX() < 0) {
            pose = pose.newX(GameDriver.SCREEN_WIDTH);
        }

        // wrap at top/bottom edge
        if (pose.getY() > GameDriver.SCREEN_HEIGHT) {
            pose = pose.newY(0);
        } else if (pose.getY() < 0) {
            pose = pose.newY(GameDriver.SCREEN_HEIGHT);
        }
    }

}
