/**
 * The ship that the player controls.
 * 
 * @author CS159 Instructors
 * @version s23
 */
public class Ship extends GameElement {

    public static final int SHIP_WIDTH = 10;
    public static final int SHIP_HEIGHT = 20;
    public static final double SHIP_TURN = 0.1;
    public static final double SHIP_MOVE = 0.1;
    public static final double FRICTION = 0.02;

    /**
     * Constructs a ship at the center of the screen and pointing upward.
     */
    public Ship() {
        super(new Pose(GameDriver.SCREEN_WIDTH / 2,
                GameDriver.SCREEN_HEIGHT / 2, Math.PI / 2.0), new Vector2D(Math.PI / 2.0, 0),
                Math.max(SHIP_WIDTH, SHIP_HEIGHT) / 2.0);
    }
    
    /**
     * Turns the ship left.
     */
    public void turnLeft() {
        pose = pose.newHeading(pose.getHeading() + SHIP_TURN);
    }
    
    /**
     * Turns the ship right.
     */
    public void turnRight() {
        pose = pose.newHeading(pose.getHeading() - SHIP_TURN);
    }
    
    /**
     * Propels the ship in the direction it is facing.
     */
    public void thrust() {
        Vector2D thrust = new Vector2D(pose.getHeading(), SHIP_MOVE);
        velocity = velocity.add(thrust);
    }
    
    @Override
    public void update() {
        super.update();
        velocity = velocity.newMagnitude(Math.max(0, velocity.getMagnitude() - FRICTION));
    }

    @Override
    public void draw() {
        GameUtils.drawPoseAsTriangle(pose, SHIP_WIDTH, SHIP_HEIGHT);
    }

}
